

#import "XPBaseActionView.h"

@interface XPBaseActionView()
@property (nonatomic, weak, readwrite) UIViewController *rootViewController;//<! 父控制器

@property (nonatomic, strong) UIButton *shadowView;//<! 遮罩view
@property (nonatomic, strong) UIImageView *contentView;//<! 内容view
@property (nonatomic, strong) UIButton *dismissButton;//<! 退出按钮

@end

@implementation XPBaseActionView


#pragma mark - attribute

#pragma mark - lifeCycle
- (instancetype)init
{
    self = [super init];
    if (self) {
        [self loadDefault];
    }
    return self;
}

- (instancetype)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if (self) {
        
    }
    return self;
}

- (instancetype)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        
    }
    return self;
}

/**
 *  @brief 初始换
 *
 *  @param rootViewController 父控制器
 *  @param view               要添加上的view,先设定好view的大小
 *  @param beforeHideBlock
 *
 *  @return
 */
- (instancetype)initWithRootViewController:(UIViewController *) rootViewController
                                      view:(UIView *) view
                           beforeHideBlock:(void(^)()) beforeHideBlock
{
    self = [self init];
    if (self) {
        self.rootViewController = rootViewController;
        if (view) {
            [self.contentView addSubview:view];
            CGRect bounds = view.bounds;
            self.contentView.bounds = bounds;
            view.frame = bounds;
        }
        self.beforeHideBlock = beforeHideBlock;
    }
    return self;
}

#pragma mark - config
- (void) loadDefault
{
    //1.自身
    CGRect screen_bounds = [[UIScreen mainScreen] bounds];
    self.view.backgroundColor = [UIColor clearColor];
    self.view.bounds = screen_bounds;
    
    //2.shadowView
    UIButton *shadowView = [self creteButtonWithTitle:nil
                                     normalBackground:nil
                                       hihgBackground:nil
                                               target:self
                                               action:@selector(hide)];
    //shadowView.userInteractionEnabled = YES;
    shadowView.backgroundColor = [UIColor colorWithRed:10/255.0 green:10/255.0 blue:10/255.0 alpha:0.5];
    [self.view addSubview:shadowView];
    self.shadowView = shadowView;
    shadowView.frame = screen_bounds;
    
    
    //3.contentView
    UIImageView *contentView = [[UIImageView alloc] init];
    contentView.layer.cornerRadius = 3;
    contentView.layer.masksToBounds = NO;
    contentView.exclusiveTouch = NO;
    contentView.userInteractionEnabled = YES;
    contentView.image = nil;
    contentView.backgroundColor = [UIColor whiteColor];
    [self.view addSubview:contentView];
    self.contentView = contentView;
    contentView.bounds = (CGRect){CGPointZero,CGSizeMake(100, 80)};
    contentView.center = CGPointMake(screen_bounds.size.width / 2, screen_bounds.size.height / 2);
    
}


#pragma mark - function
/**
 *  @brief 创建view
 *
 *  @return
 */
- (UIView *) createView
{
    UIView *view = [[UIView alloc] init];
    view.backgroundColor = [UIColor clearColor];
    return view;
}

/**
 *  @brief 创建button
 *
 *  @param title            标题
 *  @param normalBackground  常态下的背景
 *  @param hihgBackground   高亮下的背景
 *  @param target           监听对象
 *  @param action           响应方法
 *
 *  @return button
 */
- (UIButton *) creteButtonWithTitle:(NSString *) title
                   normalBackground:(UIImage *) normalBackground
                     hihgBackground:(UIImage *) hihgBackground
                             target:(id) target
                             action:(SEL) action
{
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    [button setTitle:title
            forState:UIControlStateNormal];
    [button setTitleColor:[UIColor whiteColor]
                 forState:UIControlStateNormal];
    [button setBackgroundImage:normalBackground
                      forState:UIControlStateNormal];
    [button setBackgroundImage:hihgBackground
                      forState:UIControlStateHighlighted];
    [button addTarget:target
               action:action
     forControlEvents:UIControlEventTouchUpInside];
    button.titleLabel.textAlignment = NSTextAlignmentCenter;
    button.titleLabel.adjustsFontSizeToFitWidth = YES;
    return button;
}

/**
 *  @brief 创建button
 *
 *  @return
 */
- (UILabel *) createLabel
{
    UILabel *label = [[UILabel alloc] init];
    label.backgroundColor = [UIColor clearColor];
    label.textColor = [UIColor whiteColor];
    label.adjustsFontSizeToFitWidth = YES;
    return label;
}

/**
 *  @brief 显示
 *
 *  @param animation 动画标志
 */
- (void) showWithAnimation:(BOOL) animation
{
    [self.rootViewController.view addSubview:self.view];
    [self.rootViewController addChildViewController:self];
    self.view.frame = self.rootViewController.view.bounds;
    self.view.center = self.rootViewController.view.center;
    
    if (animation == YES) {
        self.shadowView.alpha = 0;
        self.contentView.transform = CGAffineTransformMakeScale(0.7, 0.7);
        [UIView animateWithDuration:0.1f animations:^{
            self.shadowView.alpha = 1;
            self.contentView.transform = CGAffineTransformMakeScale(1.1, 1.1);
        } completion:^(BOOL finished) {
            self.contentView.transform = CGAffineTransformIdentity;
        }];
        
    } else {
        
    }
}

/**
 *  @brief 显示，默认动画 = YES
 */
- (void) show
{
    [self showWithAnimation:YES];
}

/**
 *  @brief 隐藏
 *
 *  @param animation 动画标志
 */
- (void) hideWithAnimation:(BOOL) animation
{
    if (animation == YES) {
        [UIView animateWithDuration:0.1f animations:^{
            self.shadowView.alpha = 0;
        } completion:^(BOOL completed) {
            [self hideBeforeProcess];
            
        }];
    } else {
        [self hideBeforeProcess];
    }
}

- (void) hideBeforeProcess
{
    if (_beforeHideBlock) {
        _beforeHideBlock();
    }
    
    [self.view removeFromSuperview];
    [self removeFromParentViewController];
}

/**
 *  @brief 默认动画
 */
- (void) hide
{
    [self hideWithAnimation:YES];
}

@end
